@@ -5,7 +5,7 @@
 
     LaTeX builder.
 
-    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -18,14 +18,16 @@ from docutils.io import FileOutput
 from docutils.utils import new_document
 from docutils.frontend import OptionParser
 
-from sphinx import package_dir, addnodes
+from sphinx import package_dir, addnodes, highlighting
 from sphinx.util import texescape
+from sphinx.config import string_classes, ENUM
 from sphinx.errors import SphinxError
 from sphinx.locale import _
 from sphinx.builders import Builder
 from sphinx.environment import NoUri
 from sphinx.util.nodes import inline_all_toctrees
-from sphinx.util.osutil import SEP, copyfile
+from sphinx.util.fileutil import copy_asset_file
+from sphinx.util.osutil import SEP, make_filename
 from sphinx.util.console import bold, darkgreen
 from sphinx.writers.latex import LaTeXWriter
 
@@ -36,13 +38,12 @@ class LaTeXBuilder(Builder):
     """
     name = 'latex'
     format = 'latex'
-    supported_image_types = ['application/pdf', 'image/png',
-                             'image/gif', 'image/jpeg']
-    usepackages = []
+    supported_image_types = ['application/pdf', 'image/png', 'image/jpeg']
 
     def init(self):
         self.docnames = []
         self.document_data = []
+        self.usepackages = []
         texescape.init()
 
     def get_outdated_docs(self):
@@ -77,6 +78,16 @@ class LaTeXBuilder(Builder):
                 docname = docname[:-5]
             self.titles.append((docname, entry[2]))
 
+    def write_stylesheet(self):
+        highlighter = highlighting.PygmentsBridge(
+            'latex', self.config.pygments_style, self.config.trim_doctest_flags)
+        stylesheet = path.join(self.outdir, 'sphinxhighlight.sty')
+        with open(stylesheet, 'w') as f:
+            f.write('\\NeedsTeXFormat{LaTeX2e}[1995/12/01]\n')
+            f.write('\\ProvidesPackage{sphinxhighlight}'
+                    '[2016/05/29 stylesheet for highlighting with pygments]\n\n')
+            f.write(highlighter.get_stylesheet())
+
     def write(self, *ignored):
         docwriter = LaTeXWriter(self)
         docsettings = OptionParser(
@@ -85,6 +96,7 @@ class LaTeXBuilder(Builder):
             read_config_files=True).get_default_values()
 
         self.init_document_data()
+        self.write_stylesheet()
 
         for entry in self.document_data:
             docname, targetname, title, author, docclass = entry[:5]
@@ -97,8 +109,8 @@ class LaTeXBuilder(Builder):
             self.info("processing " + targetname + "... ", nonl=1)
             toctrees = self.env.get_doctree(docname).traverse(addnodes.toctree)
             if toctrees:
-                if toctrees[0].get('maxdepth'):
-                    tocdepth = int(toctrees[0].get('maxdepth'))
+                if toctrees[0].get('maxdepth') > 0:
+                    tocdepth = toctrees[0].get('maxdepth')
                 else:
                     tocdepth = None
             else:
@@ -122,7 +134,7 @@ class LaTeXBuilder(Builder):
         tree = self.env.get_doctree(indexfile)
         contentsname = None
         for toctree in tree.traverse(addnodes.toctree):
-            if toctree['caption']:
+            if 'caption' in toctree:
                 contentsname = toctree['caption']
                 break
 
@@ -177,33 +189,118 @@ class LaTeXBuilder(Builder):
             self.info(bold('copying images...'), nonl=1)
             for src, dest in iteritems(self.images):
                 self.info(' '+src, nonl=1)
-                copyfile(path.join(self.srcdir, src),
-                         path.join(self.outdir, dest))
+                copy_asset_file(path.join(self.srcdir, src),
+                                path.join(self.outdir, dest))
             self.info()
 
         # copy TeX support files from texinputs
+        context = {'latex_engine': self.config.latex_engine}
         self.info(bold('copying TeX support files...'))
         staticdirname = path.join(package_dir, 'texinputs')
         for filename in os.listdir(staticdirname):
             if not filename.startswith('.'):
-                copyfile(path.join(staticdirname, filename),
-                         path.join(self.outdir, filename))
+                copy_asset_file(path.join(staticdirname, filename),
+                                self.outdir, context=context)
 
         # copy additional files
         if self.config.latex_additional_files:
             self.info(bold('copying additional files...'), nonl=1)
             for filename in self.config.latex_additional_files:
                 self.info(' '+filename, nonl=1)
-                copyfile(path.join(self.confdir, filename),
-                         path.join(self.outdir, path.basename(filename)))
+                copy_asset_file(path.join(self.confdir, filename), self.outdir)
             self.info()
 
         # the logo is handled differently
         if self.config.latex_logo:
-            logobase = path.basename(self.config.latex_logo)
-            logotarget = path.join(self.outdir, logobase)
             if not path.isfile(path.join(self.confdir, self.config.latex_logo)):
                 raise SphinxError('logo file %r does not exist' % self.config.latex_logo)
-            elif not path.isfile(logotarget):
-                copyfile(path.join(self.confdir, self.config.latex_logo), logotarget)
+            else:
+                copy_asset_file(path.join(self.confdir, self.config.latex_logo), self.outdir)
         self.info('done')
+
+
+def validate_config_values(app):
+    if app.config.latex_toplevel_sectioning not in (None, 'part', 'chapter', 'section'):
+        app.warn('invalid latex_toplevel_sectioning, ignored: %s' %
+                 app.config.latex_toplevel_sectioning)
+        app.config.latex_toplevel_sectioning = None
+
+    if app.config.latex_use_parts:
+        if app.config.latex_toplevel_sectioning:
+            app.warn('latex_use_parts conflicts with latex_toplevel_sectioning, ignored.')
+        else:
+            app.warn('latex_use_parts is deprecated. Use latex_toplevel_sectioning instead.')
+            app.config.latex_toplevel_sectioning = 'parts'
+
+    if app.config.latex_use_modindex is not True:  # changed by user
+        app.warn('latex_use_modeindex is deprecated. Use latex_domain_indices instead.')
+
+    if app.config.latex_preamble:
+        if app.config.latex_elements.get('preamble'):
+            app.warn("latex_preamble conflicts with latex_elements['preamble'], ignored.")
+        else:
+            app.warn("latex_preamble is deprecated. Use latex_elements['preamble'] instead.")
+            app.config.latex_elements['preamble'] = app.config.latex_preamble
+
+    if app.config.latex_paper_size != 'letter':
+        if app.config.latex_elements.get('papersize'):
+            app.warn("latex_paper_size conflicts with latex_elements['papersize'], ignored.")
+        else:
+            app.warn("latex_paper_size is deprecated. "
+                     "Use latex_elements['papersize'] instead.")
+            if app.config.latex_paper_size:
+                app.config.latex_elements['papersize'] = app.config.latex_paper_size + 'paper'
+
+    if app.config.latex_font_size != '10pt':
+        if app.config.latex_elements.get('pointsize'):
+            app.warn("latex_font_size conflicts with latex_elements['pointsize'], ignored.")
+        else:
+            app.warn("latex_font_size is deprecated. Use latex_elements['pointsize'] instead.")
+            app.config.latex_elements['pointsize'] = app.config.latex_font_size
+
+    if 'footer' in app.config.latex_elements:
+        if 'postamble' in app.config.latex_elements:
+            app.warn("latex_elements['footer'] conflicts with "
+                     "latex_elements['postamble'], ignored.")
+        else:
+            app.warn("latex_elements['footer'] is deprecated. "
+                     "Use latex_elements['preamble'] instead.")
+            app.config.latex_elements['postamble'] = app.config.latex_elements['footer']
+
+
+def setup(app):
+    app.add_builder(LaTeXBuilder)
+    app.connect('builder-inited', validate_config_values)
+
+    app.add_config_value('latex_engine',
+                         lambda self: 'pdflatex' if self.language != 'ja' else 'platex',
+                         None,
+                         ENUM('pdflatex', 'xelatex', 'lualatex', 'platex'))
+    app.add_config_value('latex_documents',
+                         lambda self: [(self.master_doc, make_filename(self.project) + '.tex',
+                                        self.project, '', 'manual')],
+                         None)
+    app.add_config_value('latex_logo', None, None, string_classes)
+    app.add_config_value('latex_appendices', [], None)
+    app.add_config_value('latex_keep_old_macro_names', True, None)
+    # now deprecated - use latex_toplevel_sectioning
+    app.add_config_value('latex_use_parts', False, None)
+    app.add_config_value('latex_toplevel_sectioning', None, None, [str])
+    app.add_config_value('latex_use_modindex', True, None)  # deprecated
+    app.add_config_value('latex_domain_indices', True, None, [list])
+    app.add_config_value('latex_show_urls', 'no', None)
+    app.add_config_value('latex_show_pagerefs', False, None)
+    # paper_size and font_size are still separate values
+    # so that you can give them easily on the command line
+    app.add_config_value('latex_paper_size', 'letter', None)
+    app.add_config_value('latex_font_size', '10pt', None)
+    app.add_config_value('latex_elements', {}, None)
+    app.add_config_value('latex_additional_files', [], None)
+
+    japanese_default = {'manual': 'jsbook',
+                        'howto': 'jreport'}
+    app.add_config_value('latex_docclass',
+                         lambda self: japanese_default if self.language == 'ja' else {},
+                         None)
+    # now deprecated - use latex_elements
+    app.add_config_value('latex_preamble', '', None)
