@@ -5,22 +5,23 @@
 
     Manual pages builder.
 
-    :copyright: Copyright 2007-2014 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 from os import path
 
+from six import string_types
 from docutils.io import FileOutput
 from docutils.frontend import OptionParser
 
 from sphinx import addnodes
-from sphinx.errors import SphinxError
 from sphinx.builders import Builder
 from sphinx.environment import NoUri
 from sphinx.util.nodes import inline_all_toctrees
+from sphinx.util.osutil import make_filename
 from sphinx.util.console import bold, darkgreen
-from sphinx.writers.manpage import ManualPageWriter, has_manpage_writer
+from sphinx.writers.manpage import ManualPageWriter
 
 
 class ManualPageBuilder(Builder):
@@ -32,15 +33,12 @@ class ManualPageBuilder(Builder):
     supported_image_types = []
 
     def init(self):
-        if not has_manpage_writer:
-            raise SphinxError('The docutils manual page writer can\'t be '
-                              'found; it is only available as of docutils 0.6.')
         if not self.config.man_pages:
             self.warn('no "man_pages" config value found; no manual pages '
                       'will be written')
 
     def get_outdated_docs(self):
-        return 'all manpages' # for now
+        return 'all manpages'  # for now
 
     def get_target_uri(self, docname, typ=None):
         if typ == 'token':
@@ -58,7 +56,7 @@ class ManualPageBuilder(Builder):
 
         for info in self.config.man_pages:
             docname, name, description, authors, section = info
-            if isinstance(authors, str):
+            if isinstance(authors, string_types):
                 if authors:
                     authors = [authors]
                 else:
@@ -73,7 +71,7 @@ class ManualPageBuilder(Builder):
             tree = self.env.get_doctree(docname)
             docnames = set()
             largetree = inline_all_toctrees(self, docnames, docname, tree,
-                                            darkgreen)
+                                            darkgreen, [docname])
             self.info('} ', nonl=True)
             self.env.resolve_references(largetree, docname, self)
             # remove pending_xref nodes
@@ -91,3 +89,13 @@ class ManualPageBuilder(Builder):
 
     def finish(self):
         pass
+
+
+def setup(app):
+    app.add_builder(ManualPageBuilder)
+
+    app.add_config_value('man_pages',
+                         lambda self: [(self.master_doc, make_filename(self.project).lower(),
+                                        '%s %s' % (self.project, self.release), [], 1)],
+                         None)
+    app.add_config_value('man_show_urls', False, None)
