@@ -5,12 +5,13 @@
 
     Texinfo builder.
 
-    :copyright: Copyright 2007-2014 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 from os import path
 
+from six import iteritems
 from docutils import nodes
 from docutils.io import FileOutput
 from docutils.utils import new_document
@@ -21,7 +22,7 @@ from sphinx.locale import _
 from sphinx.builders import Builder
 from sphinx.environment import NoUri
 from sphinx.util.nodes import inline_all_toctrees
-from sphinx.util.osutil import SEP, copyfile
+from sphinx.util.osutil import SEP, copyfile, make_filename
 from sphinx.util.console import bold, darkgreen
 from sphinx.writers.texinfo import TexinfoWriter
 
@@ -87,14 +88,14 @@ class TexinfoBuilder(Builder):
     name = 'texinfo'
     format = 'texinfo'
     supported_image_types = ['image/png', 'image/jpeg',
-                             'image/gif',]
+                             'image/gif']
 
     def init(self):
         self.docnames = []
         self.document_data = []
 
     def get_outdated_docs(self):
-        return 'all documents' # for now
+        return 'all documents'  # for now
 
     def get_target_uri(self, docname, typ=None):
         if docname not in self.docnames:
@@ -107,7 +108,7 @@ class TexinfoBuilder(Builder):
         return self.get_target_uri(to, typ)
 
     def init_document_data(self):
-        preliminary_document_data = list(map(list, self.config.texinfo_documents))
+        preliminary_document_data = [list(x) for x in self.config.texinfo_documents]
         if not preliminary_document_data:
             self.warn('no "texinfo_documents" config value found; no documents '
                       'will be written')
@@ -140,7 +141,8 @@ class TexinfoBuilder(Builder):
                 destination_path=path.join(self.outdir, targetname),
                 encoding='utf-8')
             self.info("processing " + targetname + "... ", nonl=1)
-            doctree = self.assemble_doctree(docname, toctree_only,
+            doctree = self.assemble_doctree(
+                docname, toctree_only,
                 appendices=(self.config.texinfo_appendices or []))
             self.info("writing... ", nonl=1)
             self.post_process_images(doctree)
@@ -171,14 +173,14 @@ class TexinfoBuilder(Builder):
             # fresh document
             new_tree = new_document('<texinfo output>')
             new_sect = nodes.section()
-            new_sect += nodes.title('<Set title in conf.py>',
-                                    '<Set title in conf.py>')
+            new_sect += nodes.title(u'<Set title in conf.py>',
+                                    u'<Set title in conf.py>')
             new_tree += new_sect
             for node in tree.traverse(addnodes.toctree):
                 new_sect += node
             tree = new_tree
         largetree = inline_all_toctrees(self, self.docnames, indexfile, tree,
-                                        darkgreen)
+                                        darkgreen, [indexfile])
         largetree['docname'] = indexfile
         for docname in appendices:
             appendix = self.env.get_doctree(docname)
@@ -207,7 +209,7 @@ class TexinfoBuilder(Builder):
         # copy image files
         if self.images:
             self.info(bold('copying images...'), nonl=1)
-            for src, dest in self.images.items():
+            for src, dest in iteritems(self.images):
                 self.info(' '+src, nonl=1)
                 copyfile(path.join(self.srcdir, src),
                          path.join(self.outdir, dest))
@@ -218,11 +220,25 @@ class TexinfoBuilder(Builder):
         fn = path.join(self.outdir, 'Makefile')
         self.info(fn, nonl=1)
         try:
-            mkfile = open(fn, 'w')
-            try:
+            with open(fn, 'w') as mkfile:
                 mkfile.write(TEXINFO_MAKEFILE)
-            finally:
-                mkfile.close()
         except (IOError, OSError) as err:
             self.warn("error writing file %s: %s" % (fn, err))
         self.info(' done')
+
+
+def setup(app):
+    app.add_builder(TexinfoBuilder)
+
+    app.add_config_value('texinfo_documents',
+                         lambda self: [(self.master_doc, make_filename(self.project).lower(),
+                                        self.project, '', make_filename(self.project),
+                                        'The %s reference manual.' %
+                                        make_filename(self.project),
+                                        'Python')],
+                         None)
+    app.add_config_value('texinfo_appendices', [], None)
+    app.add_config_value('texinfo_elements', {}, None)
+    app.add_config_value('texinfo_domain_indices', True, None, [list])
+    app.add_config_value('texinfo_show_urls', 'footnote', None)
+    app.add_config_value('texinfo_no_detailmenu', False, None)
