@@ -49,7 +49,7 @@
     resolved to a Python object, and otherwise it becomes simple emphasis.
     This can be used as the default role to make links 'smart'.
 
-    :copyright: Copyright 2007-2014 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -58,13 +58,20 @@ import re
 import sys
 import inspect
 import posixpath
+from six import string_types
+from types import ModuleType
 
+from six import text_type
 from docutils.parsers.rst import directives
 from docutils.statemachine import ViewList
 from docutils import nodes
 
+import sphinx
 from sphinx import addnodes
+from sphinx.util import import_object, rst
 from sphinx.util.compat import Directive
+from sphinx.pycode import ModuleAnalyzer, PycodeError
+from sphinx.ext.autodoc import Options
 
 
 # -- autosummary_toc node ------------------------------------------------------
@@ -72,18 +79,20 @@ from sphinx.util.compat import Directive
 class autosummary_toc(nodes.comment):
     pass
 
+
 def process_autosummary_toc(app, doctree):
     """Insert items described in autosummary:: to the TOC tree, but do
     not generate the toctree:: list.
     """
     env = app.builder.env
     crawled = {}
+
     def crawl_toc(node, depth=1):
         crawled[node] = True
         for j, subnode in enumerate(node):
             try:
-                if (isinstance(subnode, autosummary_toc)
-                    and isinstance(subnode[0], addnodes.toctree)):
+                if (isinstance(subnode, autosummary_toc) and
+                        isinstance(subnode[0], addnodes.toctree)):
                     env.note_toctree(env.docname, subnode[0])
                     continue
             except IndexError:
@@ -94,10 +103,12 @@ def process_autosummary_toc(app, doctree):
                 crawl_toc(subnode, depth+1)
     crawl_toc(doctree)
 
+
 def autosummary_toc_visit_html(self, node):
     """Hide autosummary toctree list in HTML output."""
     raise nodes.SkipNode
 
+
 def autosummary_noop(self, node):
     pass
 
@@ -107,6 +118,7 @@ def autosummary_noop(self, node):
 class autosummary_table(nodes.comment):
     pass
 
+
 def autosummary_table_visit_html(self, node):
     """Make the first column of the table non-breaking."""
     try:
@@ -116,8 +128,8 @@ def autosummary_table_visit_html(self, node):
             par = col1_entry[0]
             for j, subnode in enumerate(list(par)):
                 if isinstance(subnode, nodes.Text):
-                    new_text = str(subnode.astext())
-                    new_text = new_text.replace(" ", "\u00a0")
+                    new_text = text_type(subnode.astext())
+                    new_text = new_text.replace(u" ", u"\u00a0")
                     par[j] = nodes.Text(new_text)
     except IndexError:
         pass
@@ -125,9 +137,10 @@ def autosummary_table_visit_html(self, node):
 
 # -- autodoc integration -------------------------------------------------------
 
-class FakeDirective:
+class FakeDirective(object):
     env = {}
-    genopt = {}
+    genopt = Options()
+
 
 def get_documenter(obj, parent):
     """Get an autodoc.Documenter class suitable for documenting the given
@@ -138,7 +151,7 @@ def get_documenter(obj, parent):
     belongs to.
     """
     from sphinx.ext.autodoc import AutoDirective, DataDocumenter, \
-         ModuleDocumenter
+        ModuleDocumenter
 
     if inspect.ismodule(obj):
         # ModuleDocumenter.can_document_member always returns False
@@ -156,7 +169,7 @@ def get_documenter(obj, parent):
         parent_doc = parent_doc_cls(FakeDirective(), "")
 
     # Get the corrent documenter class for *obj*
-    classes = [cls for cls in list(AutoDirective._registry.values())
+    classes = [cls for cls in AutoDirective._registry.values()
                if cls.can_document_member(obj, '', False, parent_doc)]
     if classes:
         classes.sort(key=lambda cls: cls.priority)
@@ -190,8 +203,9 @@ class Autosummary(Directive):
 
     def run(self):
         self.env = env = self.state.document.settings.env
-        self.genopt = {}
+        self.genopt = Options()
         self.warnings = []
+        self.result = ViewList()
 
         names = [x.strip().split()[0] for x in self.content
                  if x.strip() and re.search(r'^[~a-zA-Z_]', x.strip()[0])]
@@ -213,7 +227,7 @@ class Autosummary(Directive):
 
             tocnode = addnodes.toctree()
             tocnode['includefiles'] = docnames
-            tocnode['entries'] = [(None, docname) for docname in docnames]
+            tocnode['entries'] = [(None, docn) for docn in docnames]
             tocnode['maxdepth'] = -1
             tocnode['glob'] = None
 
@@ -241,15 +255,21 @@ class Autosummary(Directive):
                 display_name = name.split('.')[-1]
 
             try:
-                real_name, obj, parent = import_by_name(name, prefixes=prefixes)
+                real_name, obj, parent, modname = import_by_name(name, prefixes=prefixes)
             except ImportError:
                 self.warn('failed to import %s' % name)
                 items.append((name, '', '', name))
                 continue
 
-            # NB. using real_name here is important, since Documenters
+            self.result = ViewList()  # initialize for each documenter
+            full_name = real_name
+            if not isinstance(obj, ModuleType):
+                # give explicitly separated module name, so that members
+                # of inner classes can be documented
+                full_name = modname + '::' + full_name[len(modname)+1:]
+            # NB. using full_name here is important, since Documenters
             #     handle module prefixes slightly differently
-            documenter = get_documenter(obj, parent)(self, real_name)
+            documenter = get_documenter(obj, parent)(self, full_name)
             if not documenter.parse_name():
                 self.warn('failed to parse name %s' % real_name)
                 items.append((display_name, '', '', real_name))
@@ -258,6 +278,21 @@ class Autosummary(Directive):
                 self.warn('failed to import object %s' % real_name)
                 items.append((display_name, '', '', real_name))
                 continue
+            if documenter.options.members and not documenter.check_module():
+                continue
+
+            # try to also get a source code analyzer for attribute docs
+            try:
+                documenter.analyzer = ModuleAnalyzer.for_module(
+                    documenter.get_real_modname())
+                # parse right now, to get PycodeErrors on parsing (results will
+                # be cached anyway)
+                documenter.analyzer.find_attr_docs()
+            except PycodeError as err:
+                documenter.env.app.debug(
+                    '[autodoc] module analyzer failed: %s', err)
+                # no source file -- e.g. for builtin and C modules
+                documenter.analyzer = None
 
             # -- Grab the signature
 
@@ -271,7 +306,8 @@ class Autosummary(Directive):
 
             # -- Grab the summary
 
-            doc = list(documenter.process_doc(documenter.get_doc()))
+            documenter.add_content(None)
+            doc = list(documenter.process_doc([self.result.data]))
 
             while doc and not doc[0].strip():
                 doc.pop(0)
@@ -303,7 +339,7 @@ class Autosummary(Directive):
         *items* is a list produced by :meth:`get_items`.
         """
         table_spec = addnodes.tabular_col_spec()
-        table_spec['spec'] = 'll'
+        table_spec['spec'] = 'p{0.5\linewidth}p{0.5\linewidth}'
 
         table = autosummary_table('')
         real_table = nodes.table('', classes=['longtable'])
@@ -333,7 +369,7 @@ class Autosummary(Directive):
         for name, sig, summary, real_name in items:
             qualifier = 'obj'
             if 'nosignatures' not in self.options:
-                col1 = ':%s:`%s <%s>`\ %s' % (qualifier, name, real_name, sig)
+                col1 = ':%s:`%s <%s>`\ %s' % (qualifier, name, real_name, rst.escape(sig))
             else:
                 col1 = ':%s:`%s <%s>`' % (qualifier, name, real_name)
             col2 = summary
@@ -341,6 +377,7 @@ class Autosummary(Directive):
 
         return [table_spec, table]
 
+
 def mangle_signature(sig, max_chars=30):
     """Reformat a function signature to a more compact form."""
     s = re.sub(r"^\((.*)\)$", r"\1", sig).strip()
@@ -374,7 +411,8 @@ def mangle_signature(sig, max_chars=30):
             sig += "[, %s]" % limited_join(", ", opts,
                                            max_chars=max_chars-len(sig)-4-2)
 
-    return "(%s)" % sig
+    return u"(%s)" % sig
+
 
 def limited_join(sep, items, max_chars=30, overflow_marker="..."):
     """Join a number of strings to one, limiting the length to *max_chars*.
@@ -399,6 +437,7 @@ def limited_join(sep, items, max_chars=30, overflow_marker="..."):
 
     return sep.join(list(items[:n_items]) + [overflow_marker])
 
+
 # -- Importing items -----------------------------------------------------------
 
 def get_import_prefixes_from_env(env):
@@ -408,11 +447,11 @@ def get_import_prefixes_from_env(env):
     """
     prefixes = [None]
 
-    currmodule = env.temp_data.get('py:module')
+    currmodule = env.ref_context.get('py:module')
     if currmodule:
         prefixes.insert(0, currmodule)
 
-    currclass = env.temp_data.get('py:class')
+    currclass = env.ref_context.get('py:class')
     if currclass:
         if currmodule:
             prefixes.insert(0, currmodule + "." + currclass)
@@ -421,6 +460,7 @@ def get_import_prefixes_from_env(env):
 
     return prefixes
 
+
 def import_by_name(name, prefixes=[None]):
     """Import a Python object that has the given *name*, under one of the
     *prefixes*.  The first name that succeeds is used.
@@ -432,12 +472,13 @@ def import_by_name(name, prefixes=[None]):
                 prefixed_name = '.'.join([prefix, name])
             else:
                 prefixed_name = name
-            obj, parent = _import_by_name(prefixed_name)
-            return prefixed_name, obj, parent
+            obj, parent, modname = _import_by_name(prefixed_name)
+            return prefixed_name, obj, parent, modname
         except ImportError:
             tried.append(prefixed_name)
     raise ImportError('no module named %s' % ' or '.join(tried))
 
+
 def _import_by_name(name):
     """Import a Python object given its full name."""
     try:
@@ -449,14 +490,14 @@ def _import_by_name(name):
             try:
                 __import__(modname)
                 mod = sys.modules[modname]
-                return getattr(mod, name_parts[-1]), mod
+                return getattr(mod, name_parts[-1]), mod, modname
             except (ImportError, IndexError, AttributeError):
                 pass
 
         # ... then as MODNAME, MODNAME.OBJ1, MODNAME.OBJ1.OBJ2, ...
         last_j = 0
         modname = None
-        for j in reversed(list(range(1, len(name_parts)+1))):
+        for j in reversed(range(1, len(name_parts)+1)):
             last_j = j
             modname = '.'.join(name_parts[:j])
             try:
@@ -472,9 +513,9 @@ def _import_by_name(name):
             for obj_name in name_parts[last_j:]:
                 parent = obj
                 obj = getattr(obj, obj_name)
-            return obj, parent
+            return obj, parent, modname
         else:
-            return sys.modules[modname], None
+            return sys.modules[modname], None, modname
     except (ValueError, ImportError, AttributeError, KeyError) as e:
         raise ImportError(*e.args)
 
@@ -495,7 +536,7 @@ def autolink_role(typ, rawtext, etext, lineno, inliner,
 
     prefixes = get_import_prefixes_from_env(env)
     try:
-        name, obj, parent = import_by_name(pnode['reftarget'], prefixes)
+        name, obj, parent, modname = import_by_name(pnode['reftarget'], prefixes)
     except ImportError:
         content = pnode[0]
         r[0][0] = nodes.emphasis(rawtext, content[0].astext(),
@@ -503,14 +544,28 @@ def autolink_role(typ, rawtext, etext, lineno, inliner,
     return r
 
 
+def get_rst_suffix(app):
+    def get_supported_format(suffix):
+        parser_class = app.config.source_parsers.get(suffix)
+        if parser_class is None:
+            return ('restructuredtext',)
+        if isinstance(parser_class, string_types):
+            parser_class = import_object(parser_class, 'source parser')
+        return parser_class.supported
+
+    for suffix in app.config.source_suffix:
+        if 'restructuredtext' in get_supported_format(suffix):
+            return suffix
+
+    return None
+
+
 def process_generate_options(app):
     genfiles = app.config.autosummary_generate
 
-    ext = app.config.source_suffix
-
     if genfiles and not hasattr(genfiles, '__len__'):
         env = app.builder.env
-        genfiles = [x + ext for x in env.found_docs
+        genfiles = [env.doc2path(x, base=None) for x in env.found_docs
                     if os.path.isfile(env.doc2path(x))]
 
     if not genfiles:
@@ -518,11 +573,18 @@ def process_generate_options(app):
 
     from sphinx.ext.autosummary.generate import generate_autosummary_docs
 
-    genfiles = [genfile + (not genfile.endswith(ext) and ext or '')
+    ext = app.config.source_suffix
+    genfiles = [genfile + (not genfile.endswith(tuple(ext)) and ext[0] or '')
                 for genfile in genfiles]
 
+    suffix = get_rst_suffix(app)
+    if suffix is None:
+        app.warn('autosummary generats .rst files internally. '
+                 'But your source_suffix does not contain .rst. Skipped.')
+        return
+
     generate_autosummary_docs(genfiles, builder=app.builder,
-                              warn=app.warn, info=app.info, suffix=ext,
+                              warn=app.warn, info=app.info, suffix=suffix,
                               base_path=app.srcdir)
 
 
@@ -545,4 +607,5 @@ def setup(app):
     app.add_role('autolink', autolink_role)
     app.connect('doctree-read', process_autosummary_toc)
     app.connect('builder-inited', process_generate_options)
-    app.add_config_value('autosummary_generate', [], True)
+    app.add_config_value('autosummary_generate', [], True, [bool])
+    return {'version': sphinx.__display_version__, 'parallel_read_safe': True}
