@@ -60,7 +60,7 @@ if PY3:
             raise TypeError('%r is not a Python function' % func)
         return inspect.getfullargspec(func)
 
-else:  # 2.6, 2.7
+else:  # 2.7
     from functools import partial
 
     def getargspec(func):
@@ -94,6 +94,18 @@ else:  # 2.6, 2.7
                     pass
         return inspect.ArgSpec(args, varargs, varkw, func_defaults)
 
+try:
+    import enum
+except ImportError:
+    enum = None
+
+
+def isenumattribute(x):
+    """Check if the object is attribute of enum."""
+    if enum is None:
+        return False
+    return isinstance(x, enum.Enum)
+
 
 def isdescriptor(x):
     """Check if the object is some kind of descriptor."""
@@ -110,12 +122,18 @@ def safe_getattr(obj, name, *defargs):
     except Exception:
         # sometimes accessing a property raises an exception (e.g.
         # NotImplementedError), so let's try to read the attribute directly
-        if name in obj.__dict__:
+        try:
+            # In case the object does weird things with attribute access
+            # such that accessing `obj.__dict__` may raise an exception
             return obj.__dict__[name]
+        except Exception:
+            pass
+
         # this is a catch-all for all the weird things that some modules do
         # with attribute access
         if defargs:
             return defargs[0]
+
         raise AttributeError(name)
 
 
