@@ -5,13 +5,14 @@
 
     Custom docutils writer for plain text.
 
-    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 import os
 import re
 import textwrap
 from itertools import groupby
+import warnings
 
 from six.moves import zip_longest
 
@@ -19,6 +20,7 @@ from docutils import nodes, writers
 from docutils.utils import column_width
 
 from sphinx import addnodes
+from sphinx.deprecation import RemovedInSphinx16Warning
 from sphinx.locale import admonitionlabels, _
 
 
@@ -280,8 +282,11 @@ class TextTranslator(nodes.NodeVisitor):
             char = '^'
         text = ''.join(x[1] for x in self.states.pop() if x[0] == -1)
         self.stateindent.pop()
-        self.states[-1].append(
-            (0, ['', text, '%s' % (char * column_width(text)), '']))
+        title = ['', text, '%s' % (char * column_width(text)), '']
+        if len(self.states) == 2 and len(self.states[-1]) == 0:
+            # remove an empty line before title if it is first section title in the document
+            title.pop(0)
+        self.states[-1].append((0, title))
 
     def visit_subtitle(self, node):
         pass
@@ -308,6 +313,12 @@ class TextTranslator(nodes.NodeVisitor):
         # XXX: wrap signatures in a way that makes sense
         self.end_state(wrap=False, end=None)
 
+    def visit_desc_signature_line(self, node):
+        pass
+
+    def depart_desc_signature_line(self, node):
+        self.add_text('\n')
+
     def visit_desc_name(self, node):
         pass
 
@@ -641,6 +652,10 @@ class TextTranslator(nodes.NodeVisitor):
             self.end_state(end=None)
 
     def visit_termsep(self, node):
+        warnings.warn('sphinx.addnodes.termsep will be removed at Sphinx-1.6. '
+                      'This warning is displayed because some Sphinx extension '
+                      'uses sphinx.addnodes.termsep. Please report it to '
+                      'author of the extension.', RemovedInSphinx16Warning)
         self.add_text(', ')
         raise nodes.SkipNode
 
@@ -861,6 +876,12 @@ class TextTranslator(nodes.NodeVisitor):
         if node.hasattr('explanation'):
             self.add_text(' (%s)' % node['explanation'])
 
+    def visit_manpage(self, node):
+        return self.visit_literal_emphasis(node)
+
+    def depart_manpage(self, node):
+        return self.depart_literal_emphasis(node)
+
     def visit_title_reference(self, node):
         self.add_text('*')
 
@@ -940,7 +961,9 @@ class TextTranslator(nodes.NodeVisitor):
 
     def visit_raw(self, node):
         if 'text' in node.get('format', '').split():
-            self.body.append(node.astext())
+            self.new_state(0)
+            self.add_text(node.astext())
+            self.end_state(wrap = False)
         raise nodes.SkipNode
 
     def visit_math(self, node):
